.TH CC 1 "5 November 1989" "InterViews" "InterViews Reference Manual"
.SH NAME
CC \- C++ compiler driver
.SH SYNOPSIS
.B CC
.BI [ options ]
.I file ...
.SH DESCRIPTION
The
.B CC
(capital CC) command selects at run-time which one of several C++
compilers or translators (GNU C++, cfront 2.0, or cfront 1.2) to
invoke and produces preprocessed C++ source code, C source code,
symbolic assembly language, or object code.
.PP
The
.B CC
command assumes arguments ending in
.TP
.B .c
are C++ source programs; they are compiled, and
each object program is left on the file
whose name is that of the source with
.B .o
substituted for
.BR .c .
.TP
.B .s
are assembly source programs; they are assembled into
.B .o
files.
.SH OPTIONS
The
.B CC
command accepts the following options:
.TP
.B \-n
Print what would have been done, but don't do it.
.TP
.B \-v
Verbose mode.  Print the commands as they are called.
.TP
.B \-C
Prevent
.I cpp
and
.I cfront
from removing comments.
.TP
.B \-E
Run only
.I cpp
on the
.B .c
files and send the result to standard output.
.TP
.B \-F
Run only
.I cpp
and
.I cfront
on the
.B .c
files and send the result to standard output.
.TP
.B \-Fc
Like the
.B \-F
option, but the output is C source code suitable as a
.B .c
file for
.IR cc(1) .
.TP
.BI \-. suffix
Instead of using standard output for the
.B \-E ,
.B \-F
or
.B \-Fc
options, place the output from each
.B .c
file on a file with the corresponding
.IR .suffix .
.TP
.B \-k
Keep the files __ctdt.c and __ctdt.o.
They are used to initialize global and static
class objects.  Normally they are removed.
.TP
.B \-NOMUNCH
Do not run
.I nm
and
.I munch
on the resulting object file.  This phase is normally
needed to detect static constructors and destructors.
.TP
.B \+d
Suppress expansion of inline functions.
.TP
.B \+e1,+e0
These flags affect how the compiler handles tables for
virtual functions.
.B \+e1 
causes the tables to be external and initialized; 
.B \+e0 
causes the tables to be external and uninitialized.
If neither flag is specified (the default), the tables are
local (static) and initialized.  
One file with all virtuals known to it
should be compiled with \+e1; the rest should be compiled
with \+e0.  Having one external table may significantly
reduce compile time, link time and object file size.
.TP
.B \+I
Keep C source code produced by the front end in a ..c file.
Line number information is stripped.
.TP
.B \+V
Accept regular C function declarations; use the
.I /usr/include
directory for \fB#include\fR files.
Support for this option is not guaranteed in future releases
.TP
.B +L
Generate source line number information using the format "#line %d"
instead of "#%d".
.TP
.B +x\fIfile\fR
Read a file of sizes and alignments. Each line contains three fields:
a type name, the size (in bytes), and the alignment (in bytes). This option
is useful for cross compilation.
.TP
.B \+S
Spy on
.IR cfront ;
that is, print some information on
.IR stderr .
.PP
See
.I ld(1)
for loader options,
.I as(1)
for assembler options,
.I cc(1)
for code generation options,
and
.I cpp(1)
for preprocessor options.
.SH CONFIGURATION
In addition to the above options, the
.B CC
command allows options beginning with 
.B \-CC
to override the default compilation environment:
.TP
.B \-CC_cfront1.2
.ns
.TP
.B \-CC_cfront2.0
.ns
.TP
.B \-CC_g++
Invoke cfront 1.2, cfront 2.0, or GNU C++.
.TP
.BI \-CC_cpp " prog"
.ns
.TP
.BI \-CC_include " I_flags"
.ns
.TP
.BI \-CC_compiler " prog"
.ns
.TP
.BI \-CC_translator " prog"
.ns
.TP
.BI \-CC_library " prog"
.ns
.TP
.BI \-CC_cc " prog"
.ns
.TP
.BI \-CC_as " prog"
.ns
.TP
.BI \-CC_ld " prog"
.ns
.TP
.BI \-CC_munch " prog"
.ns
.TP
.BI \-CC_mv " prog"
.ns
.TP
.BI \-CC_cp " prog"
.ns
.TP
.BI \-CC_grep " prog"
Override the default compilation environment.
.PP
To avoid giving the above options on every command line, you may set
the environment variable
.B CC_args
with the options that you would have given on the command
line. 
.SH FILES
file.c		input file
.br
file..c		cfront output
.br
file.o		object file
.br
a.out		linked output
.br
/lib/cpp		C preprocessor
.br
cfront		C++ front end
.br
/bin/cc		C compiler
.br
/lib/libc.a	standard C library; see Section (3) in the UNIX System V Programmer Reference Manual
.br
/usr/local/lib/libC.a	C++ library
.br
/usr/include/CC		standard #include file directory
.br
/usr/include			#include file directory (+V flag)
.SH "SEE ALSO"
cc(1), monitor(3), prof(1), ld(1).
.br
Bjarne Stroustrup,
.I The C++ Programming Language,
Addison-Wesley 1986.
.br
B. W. Kernighan and D. M. Ritchie,
.I The C Programming Language,
Prentice-Hall 1978.
.br
.SH DIAGNOSTICS
The diagnostics produced by
.I CC
itself are intended to be
self-explanatory.
Occasional messages may be produced by the assembler or loader.
No messages should be produced by
.IR cc(1) .
